/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/


#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <ras.h>
#include <raserror.h>
#include <tapi.h>
#ifdef UNDER_CE
#include "unimodem.h"
#endif

#ifdef UNICODE
#define _TEOF		WEOF
#else
#define _TEOF		EOF
#endif

#define _FILENAM  "RasEntry"

/*		The enumerated constants in "LabelValue" and the corresponding array  */
/* "Labels" are used to parse the RAS configuration file for matching strings.*/

enum {
	VAL_NAME,
	VAL_COUNTRYCODE,
	VAL_AREACODE,
	VAL_PHONE,
	VAL_SPECIFICIPADDR,
	VAL_IPADDR,
	VAL_SPECIFICNAMESERVERS,
	VAL_DNSADDR,
	VAL_ALTDNSADDR,
	VAL_WINSADDR,
	VAL_ALTWINSADDR,
	VAL_DEVICETYPE,
	VAL_DEVICENAME,
	VAL_IPHEADERCOMPRESSION,
	VAL_SWCOMPRESSION,
	VAL_USECOUNTRYANDAREACODES,
	VAL_REQUIREENCRYPTEDPW,
	VAL_REQUIREMSENCRYPTEDPW,
	VAL_REQUIREDATAENCRYPTION,
	VAL_DIALASLOCALCALL,
	VAL_FRAMING,
	VAL_FRAMESIZE,
	VAL_SCRIPT,
	VAL_USERNAME,
	VAL_PASSWORD,
	VAL_DOMAIN,
	VAL_NETWORKLOGON,
	VAL_PROHIBITCHAP,
	VAL_PROHIBITEAP,
	VAL_PROHIBITMSCHAP,
	VAL_PROHIBITMSCHAP2,
	VAL_PROHIBITPAP,
	VAL_DISABLELCPEXTENSIONS,
#ifdef UNDER_CE
    VAL_BAUDRATE,
#endif
	VAL_MAX
}  LabelValue;

TCHAR	*Labels[] = {
	TEXT("Name"),
	TEXT("CountryCode"),
	TEXT("AreaCode"),
	TEXT("Phone"),
	TEXT("SpecificIpAddr"),
	TEXT("IpAddr"),
	TEXT("SpecificNameServers"),
	TEXT("DnsAddr"),
	TEXT("AltDnsAddr"),
	TEXT("WinsAddr"),
	TEXT("AltWinsAddr"),
	TEXT("DeviceType"),
	TEXT("DeviceName"),
	TEXT("IpHeaderCompression"),
	TEXT("SwCompression"),
	TEXT("UseCountryAndAreaCodes"),
	TEXT("RequireEncryptedPw"),
	TEXT("RequireMsEncryptedPw"),
	TEXT("RequireDataEncryption"),
	TEXT("DialAsLocalCall"),
	TEXT("Framing"),
	TEXT("FrameSize"),
	TEXT("Script"),
	TEXT("UserName"),
	TEXT("Password"),
	TEXT("Domain"),
	TEXT("NetworkLogon"),
	TEXT("ProhibitCHAP"),
	TEXT("ProhibitEAP"),
	TEXT("ProhibitMsCHAP"),
	TEXT("ProhibitMsCHAP2"),
	TEXT("ProhibitPAP"),
	TEXT("DisableLcpExtensions")
#ifdef UNDER_CE
    , TEXT("BaudRate")
#endif
};

#define LOG_EXCEPTION          0				  /* Output verbosity levels */
#define LOG_FAIL               2
#define LOG_WARNING            3
#define LOG_ABORT              4
#define LOG_SKIP               6
#define LOG_NOT_IMPLEMENTED    8
#define LOG_PASS              10
#define LOG_DETAIL            12
#define LOG_COMMENT           14
#define LOG_MAX_VERBOSITY     15

#define MAX_LINE			 256
#define MAXENTRIES            25      /* Maximum number of phonebook entries */

HANDLE		hFile;
DWORD		dwDebugFlag = LOG_PASS;
RASENTRY    RasEntry;
#ifdef UNDER_CE
BOOL g_fUseDevConfig;
BOOL g_fDevName;
LPVARSTRING g_lpDevConfig;
DWORD g_dwDevConfigLen;

BOOL SetBaudRate(TCHAR *s);
#endif


//------------------------------------------------------------------------
//	Program functions - prototypes:
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

TCHAR GetFileChar(void);
TCHAR *GetFileLine(TCHAR *s, int cch);
void Usage(void);
RASIPADDR *GetRasIpAddress(LPTSTR lpszIpString, RASIPADDR *lpRasIpAddr);
void LogDebugData(DWORD dwVerbosity, LPCTSTR szFormat, ...);


void initRegistry()
{
	HKEY	hKey;
	WCHAR	data[MAX_PATH];
	DWORD	len, idx;
	
	RegOpenKeyEx( HKEY_CURRENT_USER, L"ControlPanel\\Dial\\Locations", 0, 0, &hKey );

	memset( data, 0, sizeof( data));
	wsprintf( data, L"Work;G;FG;,EFG; ; ;1;0" );
	len = wcslen( data );
	for ( idx=0; idx<len; idx++)
	{
		if ( data[idx] == (WCHAR)';' )
			data[idx] = 0;
	}	
	RegSetValueEx( hKey, L"0", 0, REG_MULTI_SZ, (LPBYTE)data, (len + 2)*2 );

	memset( data, 0, sizeof( data));
	wsprintf( data, L"Home;G;FG;,EFG; ; ;1;0" );
	len = wcslen( data );
	for ( idx=0; idx<len; idx++)
	{
		if ( data[idx] == (WCHAR)';' )
			data[idx] = 0;
	}
	RegSetValueEx( hKey, L"1", 0, REG_MULTI_SZ, (LPBYTE)data, (len + 2)*2 );

	RegCloseKey( hKey );
}

int _tmain(int argc, _TCHAR* argv[])
{
	DWORD           cb;
	RASDIALPARAMS	RasDialParams;
	TCHAR			szName[RAS_MaxEntryName + 1];
	TCHAR			szLine[MAX_LINE];
	TCHAR			szFileName[MAX_LINE];
	DWORD			dwLineNum = 0;
	TCHAR			*szStart, *szEnd;
	DWORD			dwLabel;
	BOOL			bolRemoveEntry = FALSE;;
	TCHAR			szEntryName[MAX_PATH];
	int				i;

	initRegistry();

    // This will create the default entries if the key does not exist.
    RasEnumEntries (NULL, NULL, NULL, &cb, NULL);

	if (argc < 2)	/* If no command-line parameters, list current entries */
	{
		LPRASENTRYNAME lpRasEntries;	// Pointer to the RasEntries.
		int            cEntries;			// Number of Entries found
		
		cb = MAXENTRIES * sizeof(RASENTRYNAME);
		lpRasEntries = (LPRASENTRYNAME)LocalAlloc(LPTR, cb);
		if (RasEnumEntries (NULL, NULL, lpRasEntries, &cb, (LPDWORD)&cEntries))
			wprintf(L"Unable to read RAS phonebook\n");
		else
		{
			wprintf(TEXT("RAS phonebook: %lu entries:\n"),  cEntries);
			for (i = 0; i < cEntries; i++)
				wprintf( TEXT("Entry #%d: \"%s\"\n"), i,  lpRasEntries[i].szEntryName);
		}
		return 0;
	}
	
	// Get a default RasEntry
	RasEntry.dwSize = sizeof(RASENTRY);
#ifdef UNDER_CE
	cb = sizeof(RASENTRY);
	RasGetEntryProperties (NULL, TEXT(""), &RasEntry, &cb, NULL, NULL);
#else
	cb = sizeof(RASENTRY) + 1000; /* NT needs space for alternate phone list */
	RasGetEntryProperties(NULL,  TEXT(""), &RasEntry, &cb, NULL, NULL);
#endif
	memset((char *)&RasDialParams, 0, sizeof(RasDialParams));
	szName[0] = TEXT('\0');
	szFileName[0] = TEXT('\0');

	// Parse the command line parameters
	memset(szEntryName,0,sizeof(szEntryName));
	for (i = 1; i < argc; ++i)
	{
		if ((_tcsnicmp(argv[i], TEXT("-d"), 2) == 0) ||
 		  (_tcsnicmp(argv[i], TEXT("/d"), 2) == 0))
		{
			if (_tcslen(argv[i]) > 2)
				dwDebugFlag = _tcstoul(argv[i] + 2, NULL, 10);
			else
				dwDebugFlag = LOG_MAX_VERBOSITY;
			wprintf( TEXT("Verbosity level = %lu\n"),  dwDebugFlag);
		}
		else if ((_tcsnicmp(argv[i], TEXT("-r"), 2) == 0) || (_tcsnicmp(argv[i], TEXT("/r"), 2) == 0))
		{
			bolRemoveEntry = TRUE;
			i++;
			if ( i < argc )
			{
				//remove entry
				wcscpy( szEntryName, argv[i]);
				if ( RasDeleteEntry( NULL, szEntryName) == 0 )
					wprintf( TEXT("Entry \"%s\" removed.\n\n"), szEntryName);

				LPRASENTRYNAME lpRasEntries;	// Pointer to the RasEntries.
				int            cEntries;			// Number of Entries found
				
				cb = MAXENTRIES * sizeof(RASENTRYNAME);
				lpRasEntries = (LPRASENTRYNAME)LocalAlloc(LPTR, cb);
				if (RasEnumEntries (NULL, NULL, lpRasEntries, &cb, (LPDWORD)&cEntries))
					wprintf(L"Unable to read RAS phonebook\n");
				else
				{
					wprintf(TEXT("RAS phonebook: %lu entries:\n"),  cEntries);
					for (i = 0; i < cEntries; i++)
						wprintf( TEXT("Entry #%d: \"%s\"\n"), i,  lpRasEntries[i].szEntryName);
				}
			}
			else 
			{
				Usage();
			}
			return 0;
		}
		else if ((*(argv[i]) == '-') || (*(argv[i]) == '/'))
		{
			Usage();
			return 0;
		}
		else if (szFileName[0] == TEXT('\0'))	   /* copy the filename over */
		{
			if (*(argv[i] + 2) == TEXT(' ')) {  /* skip over leading space */
				if (_tcslen(argv[i] + 1) < MAX_LINE) 
					_tcscpy(szFileName, argv[i] + 1);
			}
			else {
				if (_tcslen(argv[i]) < MAX_LINE) 
					_tcscpy(szFileName, argv[i]);
			}
		}
	}
	if (szFileName[0] == TEXT('\0'))
	{
		Usage();
		return 0;
	}
	wprintf( L"## RAS Connection Creation Program\n");
	wprintf( TEXT("RasEntry: verbosity = %d szFileName = \"%s\"\n"), dwDebugFlag,  szFileName);

#ifdef UNDER_CE
    g_fUseDevConfig = FALSE;
    g_fDevName = FALSE;
    g_lpDevConfig = NULL;
#endif

	hFile = CreateFile(szFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
	  OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (INVALID_HANDLE_VALUE == hFile)
	{
		wprintf ( TEXT("Error %u opening file '%s'\n"), GetLastError(), szFileName);
		return -1;
	}

	while ((TCHAR *)_TEOF != GetFileLine(szLine, MAX_LINE))
	{
		dwLineNum++;
		wprintf( TEXT("Line %d: '%s'\n"), dwLineNum, szLine);

		for (szStart = szLine; _istspace(*szStart); szStart++)
			;
		
		// Allow comment lines
		if ((TEXT(';') == *szStart) || (TEXT('#') == *szStart))
			continue;
		
		for (szEnd = szStart; *szEnd && !_istspace(*szEnd) &&
		  (TEXT('=') != *szEnd); szEnd++)
			;
		if (szEnd)
		{
			PREFAST_SUPPRESS(394, "False positive since szEnd is not past the end of the buffer");
			*szEnd = TEXT('\0');
		}
		else
		{
			wprintf( TEXT("%d : Invalid input line '%s'\n"), dwLineNum, szLine);
			continue;
		}
		for (dwLabel = 0; dwLabel != VAL_MAX; dwLabel++)
		{
			if (!_tcsicmp (Labels[dwLabel], szStart))
				break;
		}
		szStart = szEnd + 1;

		switch (dwLabel)
		{
			case VAL_NAME:
				_tcsncpy(szName, szStart, RAS_MaxEntryName);
				break;
			case VAL_COUNTRYCODE:
				RasEntry.dwCountryCode = _ttol(szStart);
				break;
			case VAL_AREACODE:
				_tcsncpy (RasEntry.szAreaCode, szStart, RAS_MaxAreaCode);
				break;
			case VAL_PHONE:
				_tcsncpy (RasEntry.szLocalPhoneNumber, szStart,
				  RAS_MaxPhoneNumber);
				break;
			case VAL_SPECIFICIPADDR:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_SpecificIpAddr;
				else
					RasEntry.dwfOptions &= ~(RASEO_SpecificIpAddr);
				break;
			case VAL_IPADDR:
				GetRasIpAddress(szStart, &(RasEntry.ipaddr));
				break;
			case VAL_SPECIFICNAMESERVERS:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_SpecificNameServers;
				else
					RasEntry.dwfOptions &= ~(RASEO_SpecificNameServers);
				break;
			case VAL_DNSADDR:
				GetRasIpAddress(szStart, &(RasEntry.ipaddrDns));
				break;
			case VAL_ALTDNSADDR:
				GetRasIpAddress(szStart, &(RasEntry.ipaddrDnsAlt));
				break;
			case VAL_WINSADDR:
				GetRasIpAddress(szStart, &(RasEntry.ipaddrWins));
				break;
			case VAL_ALTWINSADDR:
				GetRasIpAddress(szStart, &(RasEntry.ipaddrWinsAlt));
				break;
			case VAL_DEVICETYPE:
				_tcsncpy (RasEntry.szDeviceType, szStart, RAS_MaxDeviceType + 1);
				break;
			case VAL_DEVICENAME:
				_tcsncpy (RasEntry.szDeviceName, szStart, RAS_MaxDeviceName + 1);
#ifdef UNDER_CE
                g_fDevName = TRUE;
#endif
				break;
			case VAL_IPHEADERCOMPRESSION:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_IpHeaderCompression;
				else
					RasEntry.dwfOptions &= ~(RASEO_IpHeaderCompression);
				break;
			case VAL_SWCOMPRESSION:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_SwCompression;
				else
					RasEntry.dwfOptions &= ~(RASEO_SwCompression);
				break;
			case VAL_USECOUNTRYANDAREACODES:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_UseCountryAndAreaCodes;
				else
					RasEntry.dwfOptions &= ~(RASEO_UseCountryAndAreaCodes);
				break;
			case VAL_REQUIREENCRYPTEDPW:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_RequireEncryptedPw;
				else
					RasEntry.dwfOptions &= ~(RASEO_RequireEncryptedPw);
				break;
			case VAL_REQUIREMSENCRYPTEDPW:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_RequireMsEncryptedPw;
				else
					RasEntry.dwfOptions &= ~(RASEO_RequireMsEncryptedPw);
				break;
			case VAL_REQUIREDATAENCRYPTION:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_RequireDataEncryption;
				else
					RasEntry.dwfOptions &= ~(RASEO_RequireDataEncryption);
				break;
#ifdef UNDER_CE
			case VAL_DIALASLOCALCALL:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_DialAsLocalCall;
				else
					RasEntry.dwfOptions &= ~(RASEO_DialAsLocalCall);
				break;
#endif
			case VAL_FRAMING :
				if ((*szStart == 's') || (*szStart == 'S'))
					RasEntry.dwFramingProtocol = RASFP_Slip;
				break;
			case VAL_FRAMESIZE:
				RasEntry.dwFrameSize = _ttol(szStart);
				break;
			case VAL_SCRIPT :
			    _tcsncpy (RasEntry.szScript, szStart, MAX_PATH);
				break;
			case VAL_USERNAME:
				_tcsncpy (RasDialParams.szUserName, szStart, UNLEN);
				break;
			case VAL_PASSWORD:
				_tcsncpy (RasDialParams.szPassword, szStart, PWLEN);
				break;
			case VAL_DOMAIN:
			    _tcsncpy (RasDialParams.szDomain, szStart, DNLEN);
				break;
			case VAL_NETWORKLOGON:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_NetworkLogon;
				else
					RasEntry.dwfOptions &= ~(RASEO_NetworkLogon);
				break;
			case VAL_PROHIBITCHAP:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_ProhibitCHAP;
				else
					RasEntry.dwfOptions &= ~(RASEO_ProhibitCHAP);
				break;
			case VAL_PROHIBITEAP:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_ProhibitEAP;
				else
					RasEntry.dwfOptions &= ~(RASEO_ProhibitEAP);
				break;
			case VAL_PROHIBITMSCHAP:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_ProhibitMsCHAP;
				else
					RasEntry.dwfOptions &= ~(RASEO_ProhibitMsCHAP);
				break;
			case VAL_PROHIBITMSCHAP2:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_ProhibitMsCHAP2;
				else
					RasEntry.dwfOptions &= ~(RASEO_ProhibitMsCHAP2);
				break;
			case VAL_PROHIBITPAP:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_ProhibitPAP;
				else
					RasEntry.dwfOptions &= ~(RASEO_ProhibitPAP);
				break;
			case VAL_DISABLELCPEXTENSIONS:
				if ((*szStart == 'y') || (*szStart == 'Y'))
					RasEntry.dwfOptions |= RASEO_DisableLcpExtensions;
				else
					RasEntry.dwfOptions &= ~(RASEO_DisableLcpExtensions);
				break;
				
#ifdef UNDER_CE
            case VAL_BAUDRATE:
                if (g_fDevName) {
                    g_fUseDevConfig = SetBaudRate(szStart);
                }
                break;
#endif                
			default :
				wprintf( TEXT("Line %d: Unable to copy '%s' (data ='%s')\n"),
						   dwLineNum, szLine, szStart);
				break;
		}
	}

	if (szName[0] == TEXT('\0'))
	{
		wprintf( TEXT("Error: connectoid name required\n"));
		CloseHandle (hFile);
		return -2;
	}

	
	//wcscpy( RasEntry.szDeviceName, TEXT("Hayes Compatible on COM2:"));
//	wcscpy( RasEntry.szDeviceType, TEXT("modem"));
	wcscpy(RasEntry.szAreaCode,TEXT(""));
	RasEntry.dwCountryCode = 1;
	RasEntry.dwfNetProtocols = RASNP_Ip;
	RasEntry.dwFramingProtocol = RASFP_Ppp;
//	wcscpy( RasEntry.szDeviceType, RASDT_Modem);

#ifdef UNDER_CE
    if (FALSE == g_fUseDevConfig) {
        g_lpDevConfig = NULL;
        g_dwDevConfigLen = 0;
    }
	if (RasSetEntryProperties(NULL, szName, &RasEntry,
							  sizeof(RasEntry), (g_lpDevConfig) ?
							  ((LPBYTE)g_lpDevConfig + g_lpDevConfig->dwStringOffset) : NULL,
							  (g_lpDevConfig) ? g_lpDevConfig->dwStringSize : 0))
#else
	if (RasSetEntryProperties(NULL, szName, &RasEntry, sizeof(RasEntry) + 1000,
	  NULL, 0))
#endif
		wprintf( TEXT("Error %d from \"RasSetEntryProperties\"\n"),  GetLastError());
	else
		wprintf( TEXT("RasEntry '%s' created\n"), szName);

#ifdef UNDER_CE
    if (g_fUseDevConfig) {
        LocalFree(g_lpDevConfig);
    }
#endif

	RasDialParams.dwSize = sizeof(RASDIALPARAMS);
	_tcscpy (RasDialParams.szEntryName, szName);
	_tcscpy (RasDialParams.szCallbackNumber, TEXT("NULL"));
	_tcscpy (RasDialParams.szPhoneNumber, TEXT("NULL"));
	RasSetEntryDialParams(NULL, &RasDialParams, FALSE);
	
	CloseHandle (hFile);
	return 0;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Functions:
    GetFileChar and GetFileLine

Description:
    These functions are called to read, respectively, a single character or a
	line from a RAS configuration file previously opened for reading.

-------------------------------------------------------------------*/

TCHAR GetFileChar(void)
{
	unsigned char ch;
	UINT cnt;
    BOOL bRet;
	bRet = ReadFile (hFile, &ch, 1, (LPDWORD)&cnt, 0);
	if (!bRet || (0 == cnt))
		return _TEOF;
	return (TCHAR)ch;
}


TCHAR *GetFileLine(TCHAR *s, int cch)
{
	TCHAR ch;
	TCHAR *pch;

	pch = s;
	while ((ch = GetFileChar()) != _TEOF)
	{
		if (ch == TEXT('\r')|| ch == TEXT('\n'))
		{
			if (pch == s)
				continue;
			break;
		}
		else
		{
			PREFAST_SUPPRESS(394, "False positive since we pass in param cch as buffer size and do overrun check below\n");
			*pch++ = ch;
		}

		if (pch - s == (cch - 1))
			break;
	}

	if ((_TEOF == ch) && (pch == s))
		return (TCHAR *)_TEOF;

	*pch = TEXT('\0');
	return s;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Function:
    Usage

Description:
    This function prints a short list of instructions.

-------------------------------------------------------------------*/

void Usage(void)
{
	wprintf( TEXT("RasEntry utility\n"));
	wprintf( TEXT("This program will create a RAS Entry (or connectoid) "));
	wprintf( TEXT("based on the contents of a text file\n"));
	wprintf( TEXT("Usage: RasEntry [-r <EntryName>] <FileName>\n"));
	wprintf( TEXT("  Example: RasEntry \\msft.ras    ; create a phone book entry\r\n"));
	wprintf( TEXT("  Example: RasEntry -r MyCompany  ; remove a phone book entry\r\n"));
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Function:
    GetRasIpAddress

Description:
    This function reads a string containing an IP address in dotted-decimal
	format and returns it in a RASIPADDR structure; if an error occurs, NULL
	is returned.  Bytes in the structure are ordered in big-endian format for
	NT, in little-endian format for CE.

-------------------------------------------------------------------*/

RASIPADDR *GetRasIpAddress(LPTSTR lpszIpString, RASIPADDR *lpRasIpAddr)
{
	unsigned a, b, c, d;

	wprintf( TEXT("Reading IP address from \"%s\"\n"),
	  lpszIpString);
	if (_stscanf(lpszIpString, TEXT("%u.%u.%u.%u"), &a, &b, &c, &d) < 4)
		return NULL;
#ifdef UNDER_CE
	lpRasIpAddr->a = d;
	lpRasIpAddr->b = c;
	lpRasIpAddr->c = b;
	lpRasIpAddr->d = a;
#else
	lpRasIpAddr->a = a;
	lpRasIpAddr->b = b;
	lpRasIpAddr->c = c;
	lpRasIpAddr->d = d;
#endif
	return lpRasIpAddr;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Function:
    TapiCallback

Description:
    Stub because lineInitialize requires a callback function.
-------------------------------------------------------------------*/
void CALLBACK
TapiCallback(
    DWORD               hDevice,
    DWORD               dwMessage,
    DWORD               dwInstance,
    DWORD               dwParam1,
    DWORD               dwParam2,
    DWORD               dwParam3
    )
{
    return;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Function:
    FindDeviceID

Description:
    Given the device name, find the corresponding TAPI device ID.
    Set pfUnimodem to TRUE if the device is a unimodem device.
    
    Returns the TAPI device ID or -1 for failure.
-------------------------------------------------------------------*/
DWORD FindDeviceID(TCHAR * szDeviceName, BOOL * pfUnimodem)
{
    DWORD dwDeviceID;
    DWORD dwNumDevices;
    DWORD rc;
    HLINEAPP hLineApp;
    LPLINEDEVCAPS lpDevCaps;
    DWORD dwCapsLen;
    LPTSTR lpszTSP;

    wprintf( TEXT(_FILENAM) TEXT(" - FindDeviceID: %s\n"), szDeviceName);

    rc = lineInitialize(
            &hLineApp,
            (HINSTANCE)1, // hInstance is unused
            TapiCallback,
            TEXT("RASENTRY"),
            &dwNumDevices
            );
    if (rc) {
        wprintf( TEXT(_FILENAM) TEXT(" - FindDeviceID - lineInitialize failed 0x%x\n"), rc);
        return -1;
    }

    dwCapsLen = sizeof(LINEDEVCAPS);

    //
    // Loop looking for a TAPI device whose line name matches szDeviceName
    //
    for (dwDeviceID = 0; dwDeviceID < dwNumDevices; dwDeviceID++) {
        lpDevCaps = (LPLINEDEVCAPS)LocalAlloc(LPTR, dwCapsLen);
        if (NULL == lpDevCaps) {
            wprintf( TEXT(_FILENAM) TEXT(" - FindDeviceID - LocalAlloc failed\n"));
            goto fdi_fail;
        }
        lpDevCaps->dwTotalSize = dwCapsLen;
        rc = lineGetDevCaps(
                hLineApp,
                dwDeviceID,
                TAPI_CURRENT_VERSION,
                0,
                lpDevCaps
                );
        if (rc) {
            LocalFree(lpDevCaps);
            continue;
        }

        //
        // Make sure we get the full devcaps info
        //
        if (lpDevCaps->dwTotalSize < lpDevCaps->dwNeededSize) {
            dwCapsLen = lpDevCaps->dwNeededSize;
            LocalFree(lpDevCaps);
            lpDevCaps = (LPLINEDEVCAPS)LocalAlloc(LPTR, dwCapsLen);
            if (NULL == lpDevCaps) {
                wprintf( TEXT(_FILENAM) TEXT(" - FindDeviceID - LocalAlloc failed.\n"));
                goto fdi_fail;
            }
            lpDevCaps->dwTotalSize = dwCapsLen;
            rc = lineGetDevCaps(
                    hLineApp,
                    dwDeviceID,
                    TAPI_CURRENT_VERSION,
                    0,
                    lpDevCaps
                    );
            if ((rc) || (lpDevCaps->dwTotalSize < lpDevCaps->dwNeededSize)) {
                LocalFree(lpDevCaps);
                continue;
            }
        }

        //
        // Compare the TAPI device's line name.
        //
		if (!_tcscmp((LPTSTR) ((char *)lpDevCaps + lpDevCaps->dwLineNameOffset), szDeviceName)) {
            //
            // It's a match. Now see if it's a unimodem device
            //
            if (pfUnimodem) {
                *pfUnimodem = FALSE;
                if (lpDevCaps->dwProviderInfoSize) {
                    lpszTSP = (LPTSTR)((DWORD)lpDevCaps + lpDevCaps->dwProviderInfoOffset);
                    wprintf( TEXT(_FILENAM) TEXT(" - FindDeviceID - TSP for %d is %s\n"), dwDeviceID, lpszTSP);
                    if (!_tcscmp(lpszTSP, TEXT("UNIMODEM"))) {
                        *pfUnimodem = TRUE;
                    }
                }
            }
            lineShutdown(hLineApp);
            LocalFree(lpDevCaps);
            return dwDeviceID;
        }

    }   // for dwDeviceID

fdi_fail:
    wprintf( TEXT(_FILENAM) TEXT(" - FindDeviceID - %s is not a UNIMODEM device\n"), szDeviceName);
    lineShutdown(hLineApp);
    return -1;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Function:
    SetBaudRate

Description:
    Convert the string form of the baud rate setting to an int and then
    use the lineDevSpecific interface to unimodem to edit the private
    structure.
-------------------------------------------------------------------*/
BOOL SetBaudRate(TCHAR * szBaud)
{
    BOOL fUnimodem;
    DWORD dwDeviceID;
    DWORD dwNumDevices;
    DWORD rc;
    HLINEAPP hLineApp;
    HLINE hLine;
    LPVARSTRING lpDevConfig;
    DWORD dwLen;
    UNIMDM_CHG_DEVCFG UCD;

    //
    // Get device ID of the TAPI device whose line name matches the RASENTRY szDeviceName
    //
    dwDeviceID = FindDeviceID(RasEntry.szDeviceName, &fUnimodem);
    if (dwDeviceID == -1) {
        return FALSE;
    }

    //
    // This interface is only supported by unimodem
    //
    if (fUnimodem == FALSE) {
        return FALSE;
    }

    rc = lineInitialize(
            &hLineApp,
            (HINSTANCE)1, // hInstance is unused
            TapiCallback,
            TEXT("RASENTRY"),
            &dwNumDevices
            );
    if (rc) {
        wprintf( TEXT(_FILENAM) TEXT(" - SetBaudRate - lineInitialize failed 0x%x\n"), rc);
        return FALSE;
    }

    lpDevConfig = NULL;

    //
    // Open the TAPI device
    //
    rc = lineOpen(
            hLineApp,
            dwDeviceID,
            &hLine,
            TAPI_CURRENT_VERSION,
            0,
            0,
            LINECALLPRIVILEGE_MONITOR,
            LINEMEDIAMODE_DATAMODEM,
            NULL
            );
    if (rc) {
        wprintf( TEXT(_FILENAM) TEXT(" - SetBaudRate - lineOpen(%d) failed 0x%x\n"), dwDeviceID, rc);
        goto gbr_fail;
    }

    //
    // Find out how big its DevConfig is and get it.
    //
    dwLen = sizeof(VARSTRING);
    lpDevConfig = (LPVARSTRING)LocalAlloc(LPTR, dwLen);
    if (NULL == lpDevConfig) {
        wprintf( TEXT(_FILENAM) TEXT(" - SetBaudRate - LocalAlloc failed\n"));
        goto gbr_fail;
    }

    lpDevConfig->dwTotalSize = dwLen;
    rc = lineGetDevConfig(
            dwDeviceID,
            lpDevConfig,
            TEXT("comm/datamodem")
            );
    if (rc) {
        wprintf( TEXT(_FILENAM) TEXT(" - SetBaudRate 0 - lineGetDevConfig(%d) failed 0x%x\n"), dwDeviceID, rc);
        goto gbr_fail;
    }

    if (lpDevConfig->dwTotalSize < lpDevConfig->dwNeededSize) {
        dwLen = lpDevConfig->dwNeededSize;
        LocalFree(lpDevConfig);
        lpDevConfig = (LPVARSTRING)LocalAlloc(LPTR, dwLen);
        if (NULL == lpDevConfig) {
            wprintf( TEXT(_FILENAM) TEXT(" - SetBaudRate - LocalAlloc failed.\n"));
            goto gbr_fail;
        }

        lpDevConfig->dwTotalSize = dwLen;
        rc = lineGetDevConfig(
                dwDeviceID,
                lpDevConfig,
                TEXT("comm/datamodem")
                );
        if (rc) {
            wprintf( TEXT(_FILENAM) TEXT(" - SetBaudRate 1 - lineGetDevConfig(%d) failed 0x%x.\n"), dwDeviceID, rc);
            goto gbr_fail;
        }
    }

    //
    // Use the lineDevSpecific interface to unimodem to edit the baud rate
    // in the devconfig.
    //
    UCD.dwCommand = UNIMDM_CMD_CHG_DEVCFG;
    UCD.lpszDeviceClass = TEXT("comm/datamodem");
    UCD.lpDevConfig = (LPVARSTRING)lpDevConfig;
    UCD.dwOption = UNIMDM_OPT_BAUDRATE;
    UCD.dwValue  = _tcstol(szBaud, (TCHAR)0, 10);

    rc = lineDevSpecific(
            hLine,
            0,
            NULL,
            &UCD,
            sizeof(UCD)
            );
    if (!(rc & 0x80000000)) {   // Valid async ID ?
        lineShutdown(hLineApp);
        g_lpDevConfig = lpDevConfig;  // gets freed at end of main()
        g_dwDevConfigLen = lpDevConfig->dwNeededSize;
        return TRUE;
    }

    wprintf( TEXT(_FILENAM) TEXT(" - SetBaudRate 2 - lineDevSpecific(%d) failed 0x%x.\n"), dwDeviceID, rc);

gbr_fail:
    if (lpDevConfig) {
        LocalFree(lpDevConfig);
    }
    lineShutdown(hLineApp);
    return FALSE;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Function:
    LogDebugData

Description:
    This subroutine logs data to the debugger (CE) or command window (NT) if
	the requested verbosity is less than the global verbosity level.

-------------------------------------------------------------------*/
void LogDebugData(DWORD dwVerbosity, LPCTSTR szFormat, ...)
{
    TCHAR szBuffer[320];
    va_list pArgs;

	if (dwVerbosity <= dwDebugFlag)
	{
		va_start(pArgs, szFormat);
	    _vstprintf(szBuffer, szFormat, pArgs);
		va_end(pArgs);

	    _tcscat(szBuffer, TEXT("\n"));
		OutputDebugString(szBuffer);

#ifndef UNDER_CE
	    _tprintf(szBuffer);
#endif
	}
}



