/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/


#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <ras.h>
#include <raserror.h>

int _tmain(int argc, _TCHAR* argv[])
{
    DWORD dwCb = sizeof(RASDEVINFO);
    DWORD dwErr = ERROR_SUCCESS;
    DWORD dwDevices = 0;
    RASDEVINFO* lpRasDevInfo = NULL;

    //
    // Loop through in case the information from RAS changes between calls.
    //
    //
    // If the memory is allocated, free it.
    //
    if (NULL != lpRasDevInfo)
    {
        HeapFree(GetProcessHeap(), 0, lpRasDevInfo);
        lpRasDevInfo = NULL;
    }
    //
    // Allocate the size need for the RAS structure.
    //
    lpRasDevInfo = (LPRASDEVINFO)HeapAlloc(GetProcessHeap(), 0, dwCb);
    if (NULL == lpRasDevInfo)
    {
        dwErr = ERROR_NOT_ENOUGH_MEMORY;
		printf("ERROR_NOT_ENOUGH_MEMORY\n");
        return 1;
    }

    //
    // Set the structure size for version checking purposes.
    //
    lpRasDevInfo->dwSize = sizeof(RASDEVINFO);
    //
    // Call the RAS API, bail on the loop if we are successful or an unknown
    // error occurs.
    //
    dwErr = RasEnumDevices( lpRasDevInfo, &dwCb, &dwDevices);
    if (ERROR_BUFFER_TOO_SMALL != dwErr)
    {
		if (NULL != lpRasDevInfo)
		{
			HeapFree(GetProcessHeap(), 0, lpRasDevInfo);
			lpRasDevInfo = NULL;
		}
		//
		// Allocate the size need for the RAS structure.
		//
		lpRasDevInfo = (LPRASDEVINFO)HeapAlloc(GetProcessHeap(), 0, dwCb);
		if (NULL == lpRasDevInfo)
		{
			dwErr = ERROR_NOT_ENOUGH_MEMORY;
			printf("ERROR_NOT_ENOUGH_MEMORY\n");
			return 3;
		}
		dwErr = RasEnumDevices( lpRasDevInfo, &dwCb, &dwDevices);
		if (dwErr)
		{
			printf("RasEnumDevices Failed!\n");
			return 4;
		}
	}
    //
    // In the success case print the names of the devices.
    //
    if (ERROR_SUCCESS == dwErr)
    {
        DWORD i;

        printf("The following RAS capable devices were found on this machine: (%d)\n\n", dwDevices);
        for (i = 0; i < dwDevices; i++)
        {
			wprintf(L"%s(%s)\n", lpRasDevInfo[i].szDeviceName, lpRasDevInfo[i].szDeviceType);
        }
    }
    else
    {
        printf("RasEnumDevices failed: Error = %d\n", dwErr);
    }
    //
    // Free the memory if necessary.
    //
    if (NULL != lpRasDevInfo)
    {
        HeapFree(GetProcessHeap(), 0, lpRasDevInfo);
        lpRasDevInfo = NULL;
    }

	return 0;
}

