/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/


#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>

#define PARTNER_PATH	L"SOFTWARE\\MOXA\\PARTNERS"
#define PARTNER_NAME	L"PartnerName"
#define PARTNER_EMAIL	L"PartnerEMail"
#define PARTNER_NATION	L"PartnerNation"

int CreatePartner(LPWSTR company, LPWSTR name, LPWSTR email, LPWSTR nation) 
{
	LONG lRet;
	HKEY hKey = NULL;
	DWORD dwDisposition;
	WCHAR szRegPath[MAX_PATH];

	wsprintf( szRegPath, L"%s\\%s", PARTNER_PATH, company );
	lRet = RegCreateKeyEx(HKEY_LOCAL_MACHINE, szRegPath, 0, NULL, 0, 0, NULL, &hKey, &dwDisposition);
	if(ERROR_SUCCESS != lRet)
		return 1;

	lRet = RegSetValueEx(hKey, PARTNER_NAME, 0, REG_SZ, (LPBYTE)name, (wcslen(name)+1)*sizeof(WCHAR));
	if(ERROR_SUCCESS != lRet)
	{
		RegCloseKey(hKey);
		return 2;
	}

	lRet = RegSetValueEx(hKey, PARTNER_EMAIL, 0, REG_SZ, (LPBYTE)email, (wcslen(email)+1)*sizeof(WCHAR));
	if(ERROR_SUCCESS != lRet)
	{
		RegCloseKey(hKey);
		return 3;
	}

	lRet = RegSetValueEx(hKey, PARTNER_NATION, 0, REG_SZ, (LPBYTE)nation, (wcslen(nation)+1)*sizeof(WCHAR));
	if(ERROR_SUCCESS != lRet)
	{
		RegCloseKey(hKey);
		return 4;
	}
	RegCloseKey(hKey);
	
	return 0;
}


int ReadPartner(LPWSTR company) 
{
	LONG lRet;
	HKEY hKey = NULL;
	WCHAR szRegPath[MAX_PATH];
	WCHAR szName[MAX_PATH], szEMail[MAX_PATH], szNation[MAX_PATH];
	DWORD dwLen, dwType;

	wsprintf( szRegPath, L"%s\\%s", PARTNER_PATH, company );
	lRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegPath, 0, NULL, &hKey);
	if(ERROR_SUCCESS != lRet)
		return 1;

	memset(szName, 0, sizeof(szName));
	dwLen = sizeof(szName);
	lRet = RegQueryValueEx(hKey, PARTNER_NAME, NULL, &dwType, (LPBYTE)szName, &dwLen);
	if(ERROR_SUCCESS != lRet)
	{
		RegCloseKey(hKey);
		return 2;
	}

	memset(szEMail, 0, sizeof(szEMail));
	dwLen = sizeof(szEMail);
	lRet = RegQueryValueEx(hKey, PARTNER_EMAIL, NULL, &dwType, (LPBYTE)szEMail, &dwLen);
	if(ERROR_SUCCESS != lRet)
	{
		RegCloseKey(hKey);
		return 3;
	}

	memset(szNation, 0, sizeof(szNation));
	dwLen = sizeof(szNation);
	lRet = RegQueryValueEx(hKey, PARTNER_NATION, NULL, &dwType, (LPBYTE)szNation, &dwLen);
	if(ERROR_SUCCESS != lRet)
	{
		RegCloseKey(hKey);
		return 4;
	}
	wprintf(L"[%s]\nName=%s\nE-Mail=%s\nNation=%s\n", company, szName, szEMail, szNation);
	RegCloseKey(hKey);
	
	return 0;
}


int DeletePartner(LPWSTR company) 
{
	LONG lRet;
	WCHAR szRegPath[MAX_PATH];

	memset(szRegPath,0,sizeof(szRegPath));
	wsprintf( szRegPath, L"%s\\%s", PARTNER_PATH, company );
	lRet = RegDeleteKey(HKEY_LOCAL_MACHINE, szRegPath);
	if(ERROR_SUCCESS == lRet)
	{
		wprintf(L"[%s] removed.\n", company);
	}
	
	return 0;
}

int _tmain(int argc, _TCHAR* argv[])
{
	CreatePartner( L"MyCompany", L"MyLastName", L"MyEmail@some.com", L"MyNation" );
	CreatePartner( L"Company11111", L"Name2222", L"Email2222@some.com", L"Nation3333" );

	ReadPartner(L"MyCompany");
	wprintf(L"\n");
	ReadPartner(L"Company11111");
	wprintf(L"\n");

	DeletePartner(L"MyCompany");
	DeletePartner(L"Company11111");

	return 0;
}

